//  HOUSE MODEL
//
//  FILE NAME   : house.C
//  MODULE TYPE : Depth Class
//  STATUS      : In development 
//  PURPOSE     : Visual depth analisys
//  
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyright 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecutation for the new library.
//
//            1.6 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.
//
//            1.7 (Mar 16, 1998 - Oscar Peguero), 
//            Changes in loops / steps

#include "nsl_include.h"
#include "house.h"

extern void view_to_right_retina(nsl_vector&,nsl_matrix&,nsl_data&,nsl_data&,nsl_data&);
extern void view_to_left_retina(nsl_vector&,nsl_matrix&,nsl_data&,nsl_data&,nsl_data&);
extern void retina_to_accommodation(nsl_matrix&,nsl_matrix&,nsl_vector&,nsl_data&,nsl_data&,nsl_data&,nsl_data&,nsl_data&);
extern void retina_to_disparity(nsl_matrix&,nsl_vector&,nsl_vector&);

// Depth schema methods

NslDepth::NslDepth(char *str, NslModule* parent,int sizeX,int sizeY) : 
	NslModule(str, parent), 
	a("a",this,sizeX,sizeY),
	m("m",this,sizeX,sizeY), mf("mf",this,sizeX,sizeY),
	s("s",this,sizeX,sizeY),   
	u("u",this,sizeY), uf("uf",this,sizeY), 
	tu("tu",this), tm("tm",this),
	wm("Wm",this,3), kmu("kmu",this), kam("kam",this), 
	kum("kum",this), ksm("ksm",this),
	x1("x1",this), x2("x2",this)
{
}
	
void NslDepth::initRun()
{
	m = 0;
	mf = 0;
	s = 0;
	u = 0;
	uf = 0;
	kam = 0.5;
	kum = 0.6;
	ksm = 0.8;
	kmu = 1.0;
	wm[0] = 0.25; wm[1] = 0.68; wm[2] = 0.25;
	x1 = 0.1;
	x2 = 1.1;
	tu = 0.1;
	tm = 0.3;
}

void NslDepth::simRun()
{
	nslDiff(m,tm,-m+kam*a-kum*NSLrow_vec_to_mat(uf,m.get_imax())+ksm*s+wm*mf);
	mf = NSLsigma(m,x1,x2);

	nslDiff(u,tu, -u + kmu*NSLmat_to_row_vec(mf));
	uf = NSLramp(u);
}

// Retina schema methods

Retina::Retina(char *s, NslModule* parent, int sizeX, int sizeY, int sizeR) : 
	NslModule(s, parent), 
	in("in",this,sizeX,sizeY), rl("rl",this,sizeR), 
	rr("rr",this,sizeR),
	a("a",this,sizeX,sizeY), d("d",this,sizeX,sizeY),
	w("w",this), yf("yf",this), l("l",this), 
	dmax("dmax",this), sigma("sigma",this)
{
}
	
void Retina::initRun()
{
	in             = 0;
	in.elem(0,0)   = 1.0;
	in.elem(10,10) = 1.0;
		
	rr    = 0;
	rl    = 0;
	a     = 0;
	d     = 0;
	w     = 3.0;
	yf    = -10;
	l     = 22.0;
	dmax  = 0.25;
	sigma = 0.25;
	
	view_to_right_retina(rr,in,w,yf,l);
	view_to_left_retina(rl,in,w,yf,l);
	retina_to_accommodation(a,in,rr,w,yf,l,dmax,sigma);
 	retina_to_disparity(d,rr,rl);
}

// Stereo schema methods

Stereo::Stereo(char *str, NslModule* parent, 
	int sizeX, int sizeY, int sizeR) : NslModule(str, parent),
	s("depth1", this,sizeX,sizeY),
	m("depth2", this,sizeX,sizeY), 
	r("retina", this,sizeX,sizeY,sizeR)
{
	makeConn(); 
}

void Stereo::makeConn(void)
{
	connect(r.a, s.a);
	connect(r.d, m.a);
	connect(s.mf, m.s);
	connect(m.mf, s.s);
}

DepthModel::DepthModel() : NslModel("DepthModel"),
	stereo("stereo",this,11,11,21)
{
}

AslSchemaModel<DepthModel> _DepthModel("DepthModel");

