//  HOUSE MODEL
//
//  FILE NAME   : House.h
//  MODULE TYPE : Depth Class
//  STATUS      : In development 
//  PURPOSE     : Visual depth analisys
//
//  Instituto Tecnologico Autonomo de Mexico 
//  (c) Copyrigth 1996
//
//  HISTORY : Version (Date-Name) Modification, description. 
//
//            1.0 (Agu 13, 1996 - Claudia Calderas & Salvador Marmol), 
//            Start of development for distributed execution.
//
//            1.5 (Sept 22, 1997 - Salvador Marmol), 
//            Adecuation for the new library.
//
//            1.9 (Oct 23, 1997 - Oscar Peguero), 
//            Schemas using Input/Outport ports.
//
//            1.9.1 (Nov 10, 1997 - Oscar Peguero), 
//            Implementation of the stereo schema.

class NslDepth : public NslModule
{	
public:
	NslDinFloat2 a; 		// from Retina
	NslDinFloat2 s; 		// input from other depth
	NslDoutFloat2 mf; 	// firing rate layer

	NslFloat2 m; 		// membrane potential layer	
	NslFloat1 u; 		// membrane potential layer
	NslFloat1 uf; 		// firing rate layer

	NslFloat0 tu;
	NslFloat0 tm;

	NslFloat1 wm;

	NslFloat0 kmu;
	NslFloat0 kam;
	NslFloat0 kum;
	NslFloat0 ksm;

	NslFloat0 x1;
	NslFloat0 x2;

	NslDepth(char*, NslModule*,int,int);	
	~NslDepth() {}
	
	void initRun();
	void simRun();
};

class Retina : public NslModule
{	
public:
	NslDinFloat2 in; 	// network input 
	NslDoutFloat2 a; 	// accommodation layer
	NslDoutFloat2 d; 	// disparity layer

	NslFloat1 rl; 	// left retina
	NslFloat1 rr; 	// right retina

	NslFloat0 w; 	// = 3.0;	1/2 of interpupilary distance (cm)
	NslFloat0 yf;	// = -10.0;	intersection of optical axes (0,yf) (cm)
	NslFloat0 l;	// = 22.0;	interpupilary line distance from origin (cm)

	NslFloat0 dmax;	// = 0.25;	// maximum disparity
	NslFloat0 sigma;	// = 0.25;	// spread parameter

	Retina(char*,NslModule*, int, int,int);
	~Retina() {}
	
	void initRun();
};

class Stereo : public NslModule
{
public:
	NslDepth s,m;
	Retina r;
	
	Stereo(char *, NslModule*,int,int,int);
	~Stereo() {}
			
	void makeConn();
};

class DepthModel : public NslModel
{
public:
	Stereo stereo;
	
	DepthModel();
	~DepthModel() {}
};

